Package: DEoptim
Version: 2.2-8
Title: Global Optimization by Differential Evolution
Authors@R: c(person("David", "Ardia", role = c("aut"), email = "david.ardia.ch@gmail.com", comment = c(ORCID = "0000-0003-2823-782X")), 
             person("Katharine", "Mullen", role = c("aut", "cre"), email="mullenkate@gmail.com"), 
             person("Brian", "Peterson", role = "aut"), 
             person("Joshua", "Ulrich", role = "aut"), 
             person("Kris", "Boudt", role = "ctb"))
Description: Implements the Differential Evolution algorithm for global
    optimization of a real-valued function of a real-valued parameter
    vector as described in Mullen et al. (2011) <doi:10.18637/jss.v040.i06>.
Suggests: foreach, iterators, colorspace, lattice, parallelly
Depends: parallel
Imports: methods
BugReports: https://github.com/ArdiaD/DEoptim/issues
URL: https://github.com/ArdiaD/DEoptim
License: GPL (>= 2)
Repository: CRAN
Maintainer: Katharine Mullen <mullenkate@gmail.com>
NeedsCompilation: yes
Packaged: 2022-11-09 22:45:39 UTC; kmm-windog
Author: David Ardia [aut] (<https://orcid.org/0000-0003-2823-782X>),
  Katharine Mullen [aut, cre],
  Brian Peterson [aut],
  Joshua Ulrich [aut],
  Kris Boudt [ctb]
Date/Publication: 2022-11-11 18:10:09 UTC
Built: R 4.2.0; aarch64-apple-darwin20; 2023-04-01 10:32:23 UTC; unix
Archs: DEoptim.so.dSYM

Package: DEoptim
Version: 2.2-8
Title: Global Optimization by Differential Evolution
Authors@R: c(person("David", "Ardia", role = c("aut"), email = "david.ardia.ch@gmail.com", comment = c(ORCID = "0000-0003-2823-782X")), 
             person("Katharine", "Mullen", role = c("aut", "cre"), email="mullenkate@gmail.com"), 
             person("Brian", "Peterson", role = "aut"), 
             person("Joshua", "Ulrich", role = "aut"), 
             person("Kris", "Boudt", role = "ctb"))
Description: Implements the Differential Evolution algorithm for global
    optimization of a real-valued function of a real-valued parameter
    vector as described in Mullen et al. (2011) <doi:10.18637/jss.v040.i06>.
Suggests: foreach, iterators, colorspace, lattice, parallelly
Depends: parallel
Imports: methods
BugReports: https://github.com/ArdiaD/DEoptim/issues
URL: https://github.com/ArdiaD/DEoptim
License: GPL (>= 2)
Repository: CRAN
Maintainer: Katharine Mullen <mullenkate@gmail.com>
NeedsCompilation: yes
Packaged: 2022-11-09 22:45:39 UTC; kmm-windog
Author: David Ardia [aut] (<https://orcid.org/0000-0003-2823-782X>),
  Katharine Mullen [aut, cre],
  Brian Peterson [aut],
  Joshua Ulrich [aut],
  Kris Boudt [ctb]
Date/Publication: 2022-11-11 18:10:09 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 09:26:13 UTC; unix
Archs: DEoptim.so.dSYM

useDynLib(DEoptim, .registration = TRUE)
export(DEoptim, DEoptim.control)
S3method("plot", "DEoptim")
S3method("summary", "DEoptim")
importFrom("grDevices", "devAskNewPage")
importFrom("graphics", "abline", "matplot", "par", "plot", "plot.new")
importFrom("stats", "runif")
importFrom("parallel", "clusterExport", "parApply", "makeCluster",
           "clusterCall", "stopCluster")
importFrom("methods", "hasArg")
importFrom("utils", "installed.packages")

# DEoptim
[![CRAN](https://www.r-pkg.org/badges/version/DEoptim)](https://cran.r-project.org/package=DEoptim)
[![Downloads](https://cranlogs.r-pkg.org/badges/DEoptim?color=brightgreen)](https://www.r-pkg.org/pkg/DEoptim)
[![Downloads](https://cranlogs.r-pkg.org/badges/grand-total/DEoptim?color=brightgreen)](https://www.r-pkg.org/pkg/DEoptim)

`DEoptim` ([Ardia et al., 20xx](https://CRAN.R-project.org/package=DEoptim)) implements the Differential Evolution algorithm for global optimization of a real-valued function of a real-valued parameter vector. The implementation of Differential Evolution in DEoptim interfaces with C
code for efficiency.  

The latest stable version of `DEoptim` is available at [https://cran.r-project.org/package=DEoptim](https://cran.r-project.org/package=DEoptim).

The latest development version of `DEoptim` is available at [https://github.com/ArdiaD/DEoptim](https://github.com/ArdiaD/DEoptim).

## Please cite the package in publications!

By using `DEoptim` you agree to the following rules: 

1) You must cite [Mullen et al. (2011)](https://doi.org//10.18637/jss.v040.i06) in working papers and published papers that use `DEoptim`.
2) You must place the following URL in a footnote to help others find `DEoptim`: [https://CRAN.R-project.org/package=DEoptim](https://CRAN.R-project.org/package=DEoptim). 
3) You assume all risk for the use of `DEoptim`.

Mullen, K., Ardia, D., Gil, D., Windover, D., Cline, J. (2011).  
DEoptim: An R package for global optimization by Differential Evolution.  
_Journal of Statistical Software_, 40(6), 1-26.  
[https://doi.org//10.18637/jss.v040.i06](https://doi.org//10.18637/jss.v040.i06)

## Other references

Ardia, D., Mullen, K., Peterson, B.G., Ulrich, J. (20xx).  
_DEoptim: Differential Evolution in R_.  
[https://CRAN.R-project.org/package=DEoptim](https://CRAN.R-project.org/package=DEoptim)

Ardia, D., Boudt, K., Carl, P., Mullen, K., Peterson, B.G. (2010).  
Differential Evolution with DEoptim: An application to non-convex portfolio optimization.  
_R Journal_, 3(1), 27-34.  
[https://doi.org/10.32614/RJ-2011-005](https://doi.org/10.32614/RJ-2011-005)

# DEoptim

`DEoptim` implements the Differential Evolution algorithm for global optimization of a real-valued function of a
real-valued parameter vector. The implementation of Differential Evolution in DEoptim interfaces with C
code for efficiency.  Moreover, the package is self-contained and does not depend on any other packages.

## Please cite [Mullen et al. (2011)](https://doi.org//10.18637/jss.v040.i06) in working papers and published papers that use `DEoptim`.

## Papers discussing the package and its applications:  

Mullen, K., Ardia, D., Gil, D., Windover, D., Cline, J. (2011).  
DEoptim: An R package for global optimization by Differential Evolution.  
_Journal of Statistical Software_, 40(6), 1-26.  
[https://doi.org//10.18637/jss.v040.i06](https://doi.org//10.18637/jss.v040.i06)

Ardia, D., Boudt, K., Carl, P., Mullen, K., Peterson, B.G. (2010).  
Differential Evolution with DEoptim: An application to non-convex portfolio optimization.  
_R Journal_, 3(1), 27-34.  
[https://doi.org/10.32614/RJ-2011-005](https://doi.org/10.32614/RJ-2011-005)

# This benchmark script is based on one written by Dirk Eddelbuettel 
# while he was porting DEoptim to Rcpp.  His efforts in code review
# and benchmarking are greatly appreciated.
demo.DEbenchmark <- function() {
    Rosenbrock <- function(x){
        x1 <- x[1]
        x2 <- x[2]
        100 * (x2 - x1 * x1)^2 + (1 - x1)^2
    }
    
    Wild <- function(x) {       ## 'Wild' function, global minimum at about -15.81515
        sum(10 * sin(0.3 * x) * sin(1.3 * x^2) + 0.00001 * x^4 + 0.2 * x + 80)/length(x)
    }
    
    Rastrigin <- function(x) {
        sum(x+2 - 10 * cos(2*pi*x)) + 20
    }
    
    Genrose <- function(x) {        ## One generalization of the Rosenbrock banana valley function (n parameters)
        n <- length(x)
        1.0 + sum (100 * (x[-n]^2 - x[-1])^2 + (x[-1] - 1)^2)
    }

    maxIt <- 250                        # not excessive but so that we get some run-time on simple problems
    
    basicDE <- function(n, maxIt, fun) DEoptim(fn=fun, lower=rep(-25, n), upper=rep(25, n),
                control=list(NP=10*n, itermax=maxIt, trace=FALSE))#, bs=TRUE))
    adaptDE <- function(n, maxIt, fun) DEoptim(fn=fun, lower=rep(-25, n), upper=rep(25, n),
                control=list(NP=10*n, itermax=maxIt, trace=FALSE, strategy=6))#, bs=TRUE))
    
    runPair <- function(n, maxIt, fun) {
        
        gc()
        set.seed(42)
        bt <- mean(replicate(10, system.time(invisible(basicDE(n, maxIt, fun)))[3]), trim=0.1)
        
        gc()
        set.seed(42)
        ct <- mean(replicate(10, system.time(invisible(adaptDE(n, maxIt, fun)))[3]), trim=0.1)
        
        return(data.frame(DE=bt, JADE=ct))
    }
    
    cat("# At", format(Sys.time()), "\n")
    
    reps <- c(5, 10, 20)
    
    res <- rbind(
            do.call(rbind, lapply(reps, runPair, maxIt, function(...) Rosenbrock(...))),
            do.call(rbind, lapply(reps, runPair, maxIt, function(...) Rastrigin(...))),
            do.call(rbind, lapply(reps, runPair, maxIt, function(...) Wild(...))),
            do.call(rbind, lapply(reps, runPair, maxIt, function(...) Genrose(...)))
    )
    res <- rbind(res, colMeans(res))
    rownames(res) <- c( paste("Rosenbrock", reps, sep=""),
                        paste("Rastrigin", reps, sep=""),
                        paste("Wild", reps, sep=""),
                        paste("Genrose", reps, sep=""),
                        "MEANS")
    res
}

demo.DEbenchmark()

demo.DEoptim <- function(){

  'print.comments' <- function(str){
    star <- "**********"
    cat(paste("\n",star,"\n",str,"\n",star,"\n",sep=""))
  }
  
  'wait' <- function(){
    t <- readline("\nPlease 'q' to quit the demo or any other key to continue...\n")
    if (t == "q") TRUE else FALSE
  }
    
  'Rosenbrock' <- function(x){
    x1 <- x[1]
    x2 <- x[2]
    100 * (x2 - x1 * x1)^2 + (1 - x1)^2
  }

  'Wild' <- function(x)
    10 * sin(0.3*x) * sin(1.3*x^2) + 
      0.00001 * x^4 + 0.2 * x + 80

  'demo.1' <- function(){
    r <- DEoptim(Rosenbrock, rep(-10,2), rep(10,2))
    summary(r)
  }

  'demo.2' <- function(){
    r <- DEoptim(Rosenbrock, rep(-10,2), rep(10,2), 
                 control = list(NP = 100, trace = 1))
    summary(r)
  }

  'demo.3' <- function(){
    r <- DEoptim(Rosenbrock, rep(-10,2), rep(10,2), 
                 control = list(NP = 50, itermax = 300, F = 1.5, 
                   CR = 0.2, trace = 1))
    summary(r)
    plot(r, type = 'b')
  }

  'demo.4' <- function(){
    r <- DEoptim(Wild, lower = -50, upper = 50,
                 control = list(NP = 50, trace = 1))
    par(mfrow = c(2,1))
    plot(r, type = 'b')
    plot(r, plot.type = "bestvalit", type = 'l')
  }

  'demo.5' <- function(){
    r <- DEoptim(Wild, lower = -50, upper = 50,
                 control = list(NP = 50, trace = 1, digits = 8))
  }

  str.stop <- "end of the demo"
  tstr <- "\nRun the optimization process for the 'Rosenbrock'"
  tstr <- paste(tstr, "\nBanana function. Search space [-10,10]^2.\n", sep = "")
  print.comments(tstr)
  print(Rosenbrock)
  print(demo.1)
  if (wait()) stop(str.stop) else demo.1()
  
  tstr <- "\nDecrease to 100 the members in the population.\n"
  print.comments(tstr)
  print(demo.2)
  if (wait()) stop(str.stop) else demo.2()
  
  tstr <- "\nIncrease the number of iterations to 300, and"
  tstr <- paste(tstr, "\nmodify crossover and F parameters.\n", sep = "")
  tsts <- paste(tstr, "the result")
  print.comments(tstr)
  print(demo.3)
  if (wait()) stop(str.stop) else demo.3()
  
  tstr <- "\nRun the optimization process for the 'Wild' function."
  tstr <- paste(tstr, "\nSearch space [-50,50].\n", sep = "")
  print.comments(tstr)
  print(Wild)
  plot(Wild, -50, 50, n = 1000,
       main = "DEoptim minimizing 'Wild function'")
  if (wait()) stop(str.stop) else demo.4()

#  tstr <- "\nIncrease the number of printed digits"
#  print.comments(tstr)
#  if (wait()) stop(str.stop) else demo.5()
  
  cat("\n",str.stop,"\n")
}

demo.DEoptim()
  

".onLoad" <- function (lib, pkg)
{
  library.dynam(pkg, pkg, lib)
  
}
".onAttach" <- function (lib, pkg)
{
  packageStartupMessage("\nDEoptim package",
   "\nDifferential Evolution algorithm in R",
   "\nAuthors: D. Ardia, K. Mullen, B. Peterson and J. Ulrich\n")
}

'summary.DEoptim' <- function(object, ...){
  digits <- max(5, getOption('digits') - 2)

  cat("\n***** summary of DEoptim object *****",
      "\nbest member   : ", round(object$optim$bestmem, digits),
      "\nbest value    : ", round(object$optim$bestval, digits),
      "\nafter         : ", round(object$optim$iter), "generations",
      "\nfn evaluated  : ", round(object$optim$nfeval), "times",
      "\n*************************************\n")

  invisible(object)
}

plot.DEoptim <- function (x, plot.type = c("bestmemit", "bestvalit",
"storepop"), ...) {
    z <- x$member
    niter <- length(z$bestvalit)
    npar <- length(z$lower)
    nam <- names(z$lower)
    nstorepop <- length(z$storepop)
    if (identical(plot.type[1], "bestmemit")) {
        plot.new()
        par(mfrow = c(min(3, npar), 1))
        for (i in 1:npar) {
            if (identical(i%%4, 0)) {
                cat("new plot\n")
                devAskNewPage(ask = TRUE)
            }
            plot(1:niter, z$bestmemit[, i], xlim = c(1, niter),
                las = 1, xlab = "iteration", ylab = "value",
                main = nam[i], ...)
            abline(h = c(z$lower[i], z$upper[i]), col = "red")
        }
    }
    else if (identical(plot.type[1], "bestvalit")) {
        plot(1:niter, z$bestvalit, xlim = c(1, niter), las = 1,
            xlab = "iteration", ylab = "function value", main =
"convergence plot",
            ...)
    }
    else if (identical(plot.type[1], "storepop") && nstorepop > 0) {
        plot.new()
        par(mfrow = c(min(3, npar), 1))
        for (i in 1:npar) {
            if (identical(i%%4, 0)) {
                cat("new plot\n")
                devAskNewPage(ask = TRUE)
            }
            tmp <- NULL
            for (j in 1:nstorepop) {
                tmp <- cbind(tmp, z$storepop[[j]][, i])
            }
            matplot(t(tmp), col = "black", pch = 20, las = 1, xlab =
"stored population", ylab = "value",
                main = nam[i], ...)
            abline(h = c(z$lower[i], z$upper[i]), col = "red")
            par(new = FALSE)
        }
    }
    else {
        warning("'plot.type' does not correspond to any plotting type",
immediate. = TRUE)
    }
}

library(DEoptim)

# Tests for partial argument matching (#2)
# 'p' matched 'params' (prior to ae82a815751f6c1eeaef4e51430df8fcef556af9)
# 'M' matched MARGIN
# 'F' matched FUN
RosenbrockPartialMatch <-
function(x, p, M, F)
{
  x1 <- x[1]
  x2 <- x[2]
  100 * (x2 - x1 * x1)^2 + (1 - x1)^2 
}
lower <- c(-10, -10)
upper <- -lower

set.seed(21)
deCtrl <- DEoptim.control(trace = FALSE, parallelType = "parallel")
deopt <- DEoptim(RosenbrockPartialMatch, lower, upper, deCtrl,
                 p = NULL, M = NULL, F = NULL)

DEoptim.control <- function(VTR = -Inf, strategy = 2, bs = FALSE, NP = NA,
                            itermax = 200, CR = 0.5, F = 0.8, trace = TRUE,
                            initialpop = NULL, storepopfrom = itermax + 1,
                            storepopfreq = 1,  p = 0.2, c = 0,
                            reltol, steptol, parallelType = 0, cluster = NULL,
                            packages = c(), parVar = c(), 
                            foreachArgs = list() ) {
  if (itermax <= 0) {
    warning("'itermax' <= 0; set to default value 200\n", immediate. = TRUE)
    itermax <- 200
  }
  if (F < 0 | F > 2) {
    warning("'F' not in [0,2]; set to default value 0.8\n", immediate. = TRUE)
    F <- 0.8
  }
  if (CR < 0 | CR > 1) {
    warning("'CR' not in [0,1]; set to default value 0.5\n", immediate. = TRUE)
    CR <- 0.5
  }
  if (strategy < 1 | strategy > 6) {
    warning("'strategy' not in {1,...,6}; set to default value 2\n",
            immediate. = TRUE)
    strategy <- 2
  }
    
  bs <- (bs > 0)

  if ( trace < 0 ) {
    warning("'trace' cannot be negative; set to 'TRUE'")
    trace <- TRUE
  }

  storepopfreq <- floor(storepopfreq)
  if (storepopfreq > itermax)
    storepopfreq <- 1

  if (p <= 0 || p > 1) {
    warning("'p' not in (0,1]; set to default value 0.2\n", immediate. = TRUE)
    p <- 0.2
  }

  if (c < 0 || c > 1) {
    warning("'c' not in [0,1]; set to default value 0\n", immediate. = TRUE)
    c <- 0
  }

  if (missing(reltol)) {
    reltol <- sqrt(.Machine$double.eps)
  }

  if (missing(steptol)) {
    steptol <- itermax
  }

  if(!(is.null(initialpop))) {
    if(is.na(NP))
      if(is.matrix(initialpop))
        NP <- dim(initialpop)[1]
      else stop("initialpop must be a matrix")
    else
      if(NP != dim(initialpop)[1]) {
        warning("Resetting NP to the number of rows in initialpop")
        NP <- dim(initialpop)[1]
      }
  }

  list(VTR = VTR, strategy = strategy, NP = NP, itermax = itermax, CR
       = CR, F = F, bs = bs, trace = trace, initialpop = initialpop,
       storepopfrom = storepopfrom, storepopfreq = storepopfreq, p =
       p, c = c, reltol = reltol, steptol = steptol, parallelType =
           parallelType, cluster = cluster,
       packages = packages, parVar = parVar, foreachArgs =
           foreachArgs)
}

DEoptim <- function(fn, lower, upper, control = DEoptim.control(), ...,
                    fnMap=NULL) {
    if (length(lower) != length(upper))
        stop("'lower' and 'upper' are not of same length")
    if (!is.vector(lower))
        lower <- as.vector(lower)
    if (!is.vector(upper))
        upper <- as.vector(upper)
    if (any(lower > upper))
        stop("'lower' > 'upper'")
    if (any(lower == "Inf"))
        warning("you set a component of 'lower' to 'Inf'. May imply 'NaN' results", immediate. = TRUE)
    if (any(lower == "-Inf"))
        warning("you set a component of 'lower' to '-Inf'. May imply 'NaN' results", immediate. = TRUE)
    if (any(upper == "Inf"))
        warning("you set a component of 'upper' to 'Inf'. May imply 'NaN' results", immediate. = TRUE)
    if (any(upper == "-Inf"))
        warning("you set a component of 'upper' to '-Inf'. May imply 'NaN' results", immediate. = TRUE)
    if (!is.null(names(lower)))
        nam <- names(lower)
    else if (!is.null(names(upper)) & is.null(names(lower)))
        nam <- names(upper)
    else
        nam <- paste("par", 1:length(lower), sep = "")
    
    ctrl <- do.call(DEoptim.control, as.list(control))
    ctrl$npar <- length(lower)
    if(is.na(ctrl$NP))
        ctrl$NP <- 10*length(lower)
    if (ctrl$NP < 4) {
        warning("'NP' < 4; set to default value 10*length(lower)\n",
                immediate. = TRUE)
        ctrl$NP <- 10*length(lower)
    }
    if (ctrl$NP < 10*length(lower)) 
        warning("For many problems it is best to set 'NP' (in 'control') to be at least ten times the length of the parameter vector. \n", immediate. = TRUE)
    if (!is.null(ctrl$initialpop)) {
        ctrl$specinitialpop <- TRUE
        if(!identical(as.numeric(dim(ctrl$initialpop)), as.numeric(c(ctrl$NP, ctrl$npar))))
            stop("Initial population is not a matrix with dim. NP x length(upper).")
    }
    else {
        ctrl$specinitialpop <- FALSE
        ctrl$initialpop <- 0.0
    }
    ##
    ctrl$trace <- as.numeric(ctrl$trace)
    ctrl$specinitialpop <- as.numeric(ctrl$specinitialpop)
    ctrl$initialpop <- as.numeric(ctrl$initialpop)
    if(!is.null(ctrl$cluster)) { ## use provided cluster
        if(!inherits(ctrl$cluster, "cluster"))
            stop("cluster is not a 'cluster' class object")
        parallel::clusterExport(cl, ctrl$parVar)
        fnPop <- function(params, ...) {
            parallel::parApply(cl=ctrl$cluster,params,1,fn,...)
        }
    }
    else if(ctrl$parallelType == 2) { ## use foreach 
        if(!foreach::getDoParRegistered()) {
            foreach::registerDoSEQ()
        }
        args <-  ctrl$foreachArgs
        fnPop <- function(params, ...) {
            my_chunksize <- ceiling(NROW(params)/foreach::getDoParWorkers())
            my_iter <- iterators::iter(params,by="row",chunksize=my_chunksize)
            args$i <- my_iter
            args$.combine <- c
            if (!is.null(args$.export))
                args$.export = c(args$.export, "fn")
            else
                args$.export = "fn"
            if (is.null(args$.errorhandling))
                args$.errorhandling = c('stop', 'remove', 'pass')
            if (is.null(args$.verbose))
                args$.verbose = FALSE
            if (is.null(args$.inorder))
                args$.inorder = TRUE
            if (is.null(args$.multicombine))
                args$.multicombine = FALSE
            foreach::"%dopar%"(do.call(foreach::foreach, args), apply(i,1,fn,...))
      
        }
    }
    else if(ctrl$parallelType == 1){ ## use parallel 
        cl <- parallel::makeCluster(parallel::detectCores())
        packFn <- function(packages) {
            for(i in packages)
                library(i, character.only = TRUE)
        }
        parallel::clusterCall(cl, packFn, ctrl$packages)
        parallel::clusterExport(cl, ctrl$parVar)
        fnPop <- function(params, ...) {
            parallel::parApply(cl=cl,params,1,fn,...)
        }
    }
    else {  ## use regular for loop / apply
        fnPop <- function(params, ...) {
            apply(params,1,fn,...)
        }
    }
    
    ## Mapping function
    if(is.null(fnMap)) {
        fnMapC <- function(params,...) params
    } else {
        fnMapC <- function(params,...) {
            mappedPop <- t(apply(params,1,fnMap))   ## run mapping function
            if(all(dim(mappedPop) != dim(params)))  ## check results
                stop("mapping function did not return an object with ",
                     "dim NP x length(upper).")
            dups <- duplicated(mappedPop)  ## check for duplicates
            np <- NCOL(mappedPop)
            tries <- 0
      while(tries < 5 && any(dups)) {
        ##print('dups!'); flush.console()
        nd <- sum(dups)
        ## generate new random population member
        newPop <- matrix(runif(nd*np),ncol=np)
        newPop <- rep(lower,each=nd) + newPop * rep(upper-lower,each=nd)
        ## replace duplicate with _mapped_ random member
        mappedPop[dups,] <- t(apply(newPop,1,fnMap))
        dups <- duplicated(mappedPop)  ## re-check for duplicates
        tries <- tries + 1
      }
      if(tries==5)
        warning("Could not remove ",sum(dups)," duplicates from the mapped ",
                "population in 5 tries. Evaluating population with duplicates.",
                call.=FALSE, immediate.=TRUE)
      ## memcpy fails if mappedPop isn't double (need TYPEOF switch in C?)
      storage.mode(mappedPop) <- "double"
      mappedPop
    }
  }

  outC <- .Call("DEoptimC", lower, upper, fnPop, ctrl, new.env(), fnMapC, PACKAGE="DEoptim")

  if(ctrl$parallelType == 1)
    parallel::stopCluster(cl) 
  
  if (length(outC$storepop) > 0) {
    nstorepop <- floor((outC$iter - ctrl$storepopfrom) / ctrl$storepopfreq)
    storepop <- list()
    cnt <- 1
    for(i in 1:nstorepop) {
      idx <- cnt:((cnt - 1) + (ctrl$NP * ctrl$npar))
      storepop[[i]] <- matrix(outC$storepop[idx], nrow = ctrl$NP, ncol = ctrl$npar,
                         byrow = TRUE)
      cnt <- cnt + (ctrl$NP * ctrl$npar)
      dimnames(storepop[[i]]) <- list(1:ctrl$NP, nam)
    }
  }
  else {
    storepop = NULL
  }

  names(outC$bestmem) <- nam
  iter <- max(1,as.numeric(outC$iter))

  names(lower) <- names(upper) <- nam
  bestmemit <- matrix(outC$bestmemit[1:(iter * ctrl$npar)],
                      nrow = iter, ncol = ctrl$npar, byrow = TRUE)

  dimnames(bestmemit) <- list(1:iter, nam)
  storepop <- as.list(storepop)

  outR <- list(optim = list(
                 bestmem = outC$bestmem,
                 bestval = outC$bestval,
                 nfeval = outC$nfeval,
                 iter = outC$iter),
               member = list(
                 lower = lower,
                 upper = upper,
                 bestmemit = bestmemit,
                 bestvalit = outC$bestvalit,
                 pop = t(outC$pop),
                 storepop = storepop))
  
  attr(outR, "class") <- "DEoptim"
  return(outR)
}

### R code from vignette source 'DEoptim.Rnw'

###################################################
### code chunk number 1: opt
###################################################
options(prompt = "R> ")


###################################################
### code chunk number 2: rast
###################################################
rastrigin <- function(x) 
  10*length(x)+sum(x^2-10*cos(2*pi*x))


###################################################
### code chunk number 3: figure1-code
###################################################
library("colorspace")
library("grid")
library("lattice")
jpeg("Rastrigin1.jpg")
x <- y <- seq(-5,5,by=.1)
z <- matrix(nrow=length(x), ncol=length(y))
for(i in 1:length(x)) {
  for(j in 1:length(y)) 
    z[i,j] <- rastrigin(c(x[i],y[j]))
}
xx <- list(fontsize=list(text=15,points=10),
           par.xlab.text=list(cex=2),
           par.ylab.text=list(cex=2),
           axis.text=list(cex=2),
           par.main.text=list(cex=2),
           layout.widths=list(left.padding=.1, right.padding=.1,
             between=0),
           layout.heights=list(top.padding=.1, bottom.padding=.1,
             between=0)
           )
levelplot(z, row.values=x,column.values=y,
          col.regions=sequential_hcl(300), xlab=expression(x[1]), 
          ylab=expression(x[2]),
          par.settings=xx,
          panel=function(z,row.values,column.values,...){
            panel.levelplot(z,row.values,column.values,...);
            panel.points(0,0,pch=21,col="white",cex=2)})

dev.off()
set.seed(123) 


###################################################
### code chunk number 4: prelim
###################################################
library("DEoptim")


###################################################
### code chunk number 5: opt
###################################################
est.ras <- DEoptim(rastrigin,lower=c(-5,-5),upper=c(5,5),
                   control=list(storepopfrom=1, trace=FALSE))


###################################################
### code chunk number 6: figure2-code
###################################################
pushLayout <- function(nr, nc, name="layout") {
  pushViewport(viewport(layout=grid.layout(nr, nc,
                          just="left", widths=unit(rep(2, nc), "null")),
                        name=name))
  for (i in 1:nr) {
    for (j in 1:nc) {
      pushViewport(viewport(layout.pos.row=i, layout.pos.col=j))
      upViewport()
    }
  }
  upViewport()
}
names.vpPath <- names.viewport <- function(x) x$name

with.vpPath <- with.viewport <- function(data, expr, ...) {
  depth <- if (data$name == "ROOT") 0 else downViewport(names(data))
  result <- eval.parent(substitute(expr))
  upViewport(depth)
  invisible(result)
}

getChildren.viewport <- function(x) x$children  

## end functions for making the plots with lattice 
## specify number of cells to fill and number of rows

n <- 6
nr <- 2
nc <- ceiling(n/nr)
xy <- list(fontsize=list(text=12,points=10),
           par.xlab.text=list(cex=1.5),
           par.ylab.text=list(cex=1.5),
           axis.text=list(cex=1.5),
           par.main.text=list(cex=1.5),
           layout.widths=list(left.padding=.1, right.padding=.1,between=0),
           layout.heights=list(top.padding=.1, bottom.padding=.1,between=0))

jpeg("Rastrigin2.jpg")

grid.newpage()
downViewport(pushLayout(nr, nc))
vpt <- current.vpTree(all=FALSE)
plotat <- c(seq(10,50,by=10),1) 
## something strange with Sweave/grid interaction, gen.1 is getting 
## placed in wrong viewport; 'fixed' by permuting plotat above

for(k in 1:n) {
  i <- plotat[k]
  with(getChildren.viewport(vpt)[[k]],
       print(levelplot(z, row.values=x,column.values=y,
                       xlab=expression(x[1]), 
                       ylab=expression(x[2]), colorkey=FALSE,
                       par.settings=xy,between = list(x = .2),
                       col.regions=sequential_hcl(300),
                       main=paste("Generation",i), 
                       panel=function(z,row.values,column.values,...){
                         panel.levelplot(z,row.values,column.values,...);
                         panel.points(est.ras$member$storepop[[i]],
                                      pch=21,fill="black",col=1,cex=.5);
                         panel.points(0,0,pch=21,col="white",cex=1)}),
             newpage = FALSE))
}
dev.off() 


###################################################
### code chunk number 7: opt
###################################################
options(prompt = "R> ")


###################################################
### code chunk number 8: ban
###################################################
genrose.f <- function(x){
n <- length(x)
fval <- 1.0 + sum (100 * (x[1:(n-1)]^2 - x[2:n])^2 + (x[2:n] - 1)^2)
return(fval)
}


###################################################
### code chunk number 9: ban1
###################################################
n <- 10
ans <- DEoptim(fn=genrose.f, lower=rep(-5, n), upper=rep(5, n),
               control=list(NP=100, itermax=4000,trace=FALSE))


###################################################
### code chunk number 10: ban2
###################################################
ans1 <- optim(par=runif(10,-5,5), fn=genrose.f, method="BFGS",
              control=list(maxit=4000))
              

\documentclass[nojss,shortnames]{jss}
\usepackage{amsmath,amssymb} 
\usepackage{xspace}
%% need no \usepackage{Sweave}

\author{David Ardia \\ aeris CAPITAL AG \And Katharine M. Mullen \\ NIST 
  \And Joshua Ulrich \\ FOSS Trading \And Brian G. Peterson \\ DV Trading}

\title{\pkg{DEoptim}: An \proglang{R} Package for Global 
  Optimization by Differential Evolution}

\Plainauthor{David Ardia, Katharine M. Mullen, Joshua Ulrich, Brian Peterson} 
\Plaintitle{DEoptim: An R Package for Global Optimization by Differential Evolution} 
\Shorttitle{DEoptim: An R Package for Differential Evolution} 

\Abstract{This introduction to the \proglang{R} package \pkg{DEoptim} is 
  an abreviated version of the manuscript 
  \citet{DEoptimJSS2011}, 
  published in the \emph{Journal of Statistical Software}. 
  \pkg{DEoptim} implements the Differential Evolution algorithm
  for global optimization of a real-valued function of a 
  real-valued parameter vector. The implementation of Differential 
  Evolution in \pkg{DEoptim} interfaces with \proglang{C} code for efficiency. Moreover, the package
  is self-contained and does not depend on any other packages. 
}
\Keywords{global optimization, evolutionary algorithm, Differential Evolution, \proglang{R} software}
\Plainkeywords{global optimization, evolutionary algorithm, Differential Evolution, R software} 

\Address{
David Ardia\\
aeris CAPITAL AG, Switzerland\\
URL:~\url{http://perso.unifr.ch/david.ardia/}  \\
  
Katharine Mullen \\ 
Ceramics Division, National Institute of Standards and Technology (NIST)\\ 
100 Bureau Drive, MS 8520, Gaithersburg, MD, 20899, USA\\ 
E-mail: \email{Katharine.Mullen@nist.gov}\\

Joshua Ulrich \\ 
FOSS Trading \\ 
URL:~\url{http://fosstrading.com}  \\

Brian G. Peterson\\
DV Trading\\
E-mail: \email{brian@braverock.com}  \\ 

}

\begin{document}

\SweaveOpts{engine=R,eps=FALSE}
%\VignetteIndexEntry{DEoptim: An R Package for Differential Evolution}
%\VignetteKeywords{global optimization, evolutionary algorithm, Differential Evolution, R}
%\VignettePackage{DEoptim}

\section{Introduction}\label{intro}

Optimization algorithms inspired by the process of natural selection
have been in use since the 1950s \citep{mitchell}, and are often
referred to as {\sl evolutionary algorithms}.  The genetic algorithm
is one such method, and was invented by John Holland in the 1960s 
\citep{holland}. Genetic algorithms apply logical operations, usually on bit strings of 
fixed or variable length,
in order to perform crossover, mutation, and selection on a
population.  Over the course of successive generations, the members of
the population are more likely to represent a minimum of an objective
function.  Genetic algorithms have proven themselves
to be useful heuristic methods for global optimization, in particular
for combinatorial optimization problems.  Evolution strategies are
another variety of evolutionary algorithm, in which members of the
population are represented with floating point numbers, and the
population is transformed over successive generations using arithmetic
operations. See \citet[Section~1.2.3]{Price:Storn:Lampinen:06} for a detailed overview 
of evolutionary algorithms.

In the 1990s Rainer Storn and Kenneth Price developed an evolution strategy they termed
{\sl Differential Evolution} (DE) \citep{price}. DE is particularly
well-suited to find the global optimum of a real-valued 
function of real-valued parameters, and does not require that the function be
either continuous or differentiable.  In the roughly fifteen years
since its invention, DE has been successfully applied in a wide
variety of fields, from computational physics to operations research,
as \citet{Price:Storn:Lampinen:06} catalogue.

The \pkg{DEoptim} \citep{DEoptim} implementation of DE was motivated by 
our desire to extend the set of algorithms
available for global optimization  in the 
\proglang{R} language and environment for statistical computing 
\citep{r_env}. \pkg{DEoptim} has been published on the Comprehensive
\proglang{R} Archive Network and is available at
\url{http://cran.r-project.org/web/packages/DEoptim/}. The \pkg{DEoptim} project 
is hosted on \proglang{R}-forge at \url{https://r-forge.r-project.org/projects/deoptim/}.
Since becoming publicly available it has 
been used by a variety of authors, e.g., 
\citet{Borner:Higgins:Kantelhardt:Scheiter:07},
\citet{Higgins:Kantelhardt:Scheiter:Boerner:07},
\citet{OpsinaArango:09}, 
\citet{ArdiaBoudtCarlMullenPeterson2011} and \citet{Ardia:Ospina:Giraldo:2011} to 
solve optimization problems arising in diverse domains. Interested readers
are referred to \citet{DEoptimJSS2011} for a 
longer version of the present vignette. 
  
The recently released \pkg{RcppDE} \citep{RcppDE} package ports the \proglang{C}
code in \pkg{DEoptim} to \proglang{C++}. The documentation claims \pkg{RcppDE} is more
efficient than the \proglang{C}-based implementation in \pkg{DEoptim} version 2.0-7, but most
of the increased efficiency is attributable to a non-trivial
difference in the functionality: \pkg{RcppDE} does not pass \code{"..."} to the
objective function.  \pkg{DEoptim} version 2.0-8 incorporates several
language-independent improvements made in \pkg{RcppDE} and compares
favorably to a patched version of \pkg{RcppDE} that passes \code{"..."}.


We hope that the \proglang{R} package \pkg{DEoptim} will be fruitful 
for many users. If you use \proglang{R} or \pkg{DEoptim}, please cite the software in publications.

In the remainder of this vignette we elaborate on \pkg{DEoptim}'s
implementation and use. In Section~\ref{inex}, the package 
is introduced via a simple example.  Section~\ref{alg}
describes the underlying algorithm.  Section~\ref{imp} describes the 
\proglang{R} implementation and serves as a
user manual.

\subsection{An introductory example}\label{inex} 

Minimization of the Rastrigin function in $x \in \Re^D$
\begin{align*} 
  f(x) = \sum_{j=1}^{D} \left( x_j^2 - 10 \cos \left( 2 \pi x_j \right) 
  + 10 \right) 
\end{align*}
for $D=2$ is a common test for global optimization algorithms.  

This function is possible to represent in \proglang{R} as 

<<opt,echo=FALSE,results=hide>>=
options(prompt = "R> ")
@ 
<<rast>>=
rastrigin <- function(x) 
  10*length(x)+sum(x^2-10*cos(2*pi*x))
@ 
As shown in Figure~\ref{rast}, for $D=2$
the function has a global minimum $f(x)=0$ at the point $(0,0)$.  

<<figure1-code,echo=FALSE,results=hide>>=
library("colorspace")
library("grid")
library("lattice")
jpeg("Rastrigin1.jpg")
x <- y <- seq(-5,5,by=.1)
z <- matrix(nrow=length(x), ncol=length(y))
for(i in 1:length(x)) {
  for(j in 1:length(y)) 
    z[i,j] <- rastrigin(c(x[i],y[j]))
}
xx <- list(fontsize=list(text=15,points=10),
           par.xlab.text=list(cex=2),
           par.ylab.text=list(cex=2),
           axis.text=list(cex=2),
           par.main.text=list(cex=2),
           layout.widths=list(left.padding=.1, right.padding=.1,
             between=0),
           layout.heights=list(top.padding=.1, bottom.padding=.1,
             between=0)
           )
levelplot(z, row.values=x,column.values=y,
          col.regions=sequential_hcl(300), xlab=expression(x[1]), 
          ylab=expression(x[2]),
          par.settings=xx,
          panel=function(z,row.values,column.values,...){
            panel.levelplot(z,row.values,column.values,...);
            panel.points(0,0,pch=21,col="white",cex=2)})

dev.off()
set.seed(123) 
@


\begin{figure}[t!]
\centering
\includegraphics[width=.5\textwidth]{Rastrigin1.jpg}
\caption{A contour plot of the two-dimensional Rastrigin function $f(x)$.  
The global minimum $f(x)=0$ is at $(0,0)$ and is marked with an open 
white circle.}
\label{rast} 
\end{figure}

In order to minimize this function using \pkg{DEoptim}, 
the \proglang{R} interpreter is
invoked, and the package is loaded with the command 
<<prelim>>=
library("DEoptim")
@
The \code{DEoptim} function of the package \pkg{DEoptim}
searches for minima of the objective function between
lower and upper bounds on each parameter to be optimized.  Therefore
in the call to \code{DEoptim} we specify vectors that comprise the
lower and upper bounds; these vectors are the same length as the
parameter vector.  The call to \code{DEoptim} can be made as
<<opt>>=
est.ras <- DEoptim(rastrigin,lower=c(-5,-5),upper=c(5,5),
                   control=list(storepopfrom=1, trace=FALSE))
@ 


\begin{figure}[t!] 
  \centering
  \includegraphics[width=.8\textwidth]{Rastrigin2.jpg} 
  \caption{The population associated with various generations of a call
    to \code{DEoptim} as it searches for the minimum of the Rastrigin
    function (marked with an open white circle).  The minimum is consistently 
    determined within 200 generations using the default settings of 
    \code{DEoptim}.}
  \label{ex11}
\end{figure}


Note that the vector of parameters to be optimized must be the first
argument of the objective function \code{fn} passed to \code{DEoptim}.
The above call specifies the objective function to minimize,
\code{rastrigin}, the lower and upper bounds on the the parameters,
and, via the \code{control} argument, that we want to store
intermediate populations from the first generation onwards
(\code{storepopfrom = 1}), and do not want to print out progress
information each generation (\code{trace = FALSE}).  Storing
intermediate populations allows us to examine the progress of the
optimization in detail.  
Upon initialization, the population is
comprised of 50 vectors $x$ of length two 
(50 being the default value of \code{NP}), with $x_i$ a random value 
drawn from the uniform distribution over the values defined by the 
associated lower and upper bound. The operations of
crossover, mutation, and selection explained in Section~\ref{alg}
transform the population so that the members of successive generations
are more likely to represent the global minimum of the objective
function.  The members of the population generated by the above call 
are plotted at the end of
different generations in Figure~\ref{ex11}. 
\code{DEoptim} consistently finds the minimum of the function within 200
generations using the default settings.  We have observed that \pkg{DEoptim}
solves the Rastrigin problem more efficiently than  
the simulated annealing method found in
the \proglang{R} function \code{optim}. 

<<figure2-code,echo=FALSE,results=hide>>=
pushLayout <- function(nr, nc, name="layout") {
  pushViewport(viewport(layout=grid.layout(nr, nc,
                          just="left", widths=unit(rep(2, nc), "null")),
                        name=name))
  for (i in 1:nr) {
    for (j in 1:nc) {
      pushViewport(viewport(layout.pos.row=i, layout.pos.col=j))
      upViewport()
    }
  }
  upViewport()
}
names.vpPath <- names.viewport <- function(x) x$name

with.vpPath <- with.viewport <- function(data, expr, ...) {
  depth <- if (data$name == "ROOT") 0 else downViewport(names(data))
  result <- eval.parent(substitute(expr))
  upViewport(depth)
  invisible(result)
}

getChildren.viewport <- function(x) x$children  

## end functions for making the plots with lattice 
## specify number of cells to fill and number of rows

n <- 6
nr <- 2
nc <- ceiling(n/nr)
xy <- list(fontsize=list(text=12,points=10),
           par.xlab.text=list(cex=1.5),
           par.ylab.text=list(cex=1.5),
           axis.text=list(cex=1.5),
           par.main.text=list(cex=1.5),
           layout.widths=list(left.padding=.1, right.padding=.1,between=0),
           layout.heights=list(top.padding=.1, bottom.padding=.1,between=0))

jpeg("Rastrigin2.jpg")

grid.newpage()
downViewport(pushLayout(nr, nc))
vpt <- current.vpTree(all=FALSE)
plotat <- c(seq(10,50,by=10),1) 
## something strange with Sweave/grid interaction, gen.1 is getting 
## placed in wrong viewport; 'fixed' by permuting plotat above

for(k in 1:n) {
  i <- plotat[k]
  with(getChildren.viewport(vpt)[[k]],
       print(levelplot(z, row.values=x,column.values=y,
                       xlab=expression(x[1]), 
                       ylab=expression(x[2]), colorkey=FALSE,
                       par.settings=xy,between = list(x = .2),
                       col.regions=sequential_hcl(300),
                       main=paste("Generation",i), 
                       panel=function(z,row.values,column.values,...){
                         panel.levelplot(z,row.values,column.values,...);
                         panel.points(est.ras$member$storepop[[i]],
                                      pch=21,fill="black",col=1,cex=.5);
                         panel.points(0,0,pch=21,col="white",cex=1)}),
             newpage = FALSE))
}
dev.off() 
@ 

We note that as the dimensionality of the Rastrigin problem increases, 
\pkg{DEoptim} may not be able to find the global minimum in the default 
number of generations.  Heuristics to help ensure that the global minimum is 
found include re-running the problem with a larger population size (value
of \code{NP}), and increasing the maximum allowed number of generations.  

\subsection{Problems suitable for DE}\label{inex1} 

Differential Evolution does not require derivatives of the objective
function.  It is therefore useful in situations in which the objective
function is stochastic, noisy, or difficult to differentiate.  DE,
however, may be inefficient on smooth functions, where
derivative-based methods generally are most efficient.

In the example below, a generalized Rosenbrock function is considered.
This function is differentiable and has a single local minimum.  It is
often more efficient to apply methods other than DE to optimization of
such functions.  
Functions that are smooth but have many local minima, however, 
may still be good candidates for optimization with DE, since
alternative algorithms for local, as opposed to global, optimization 
may converge to a sub-optimal solution.

A generalized Rosenbrock function is possible 
to represent in \proglang{R} as 
<<opt,echo=FALSE,results=hide>>=
options(prompt = "R> ")
@ 
<<ban>>=
genrose.f <- function(x){
n <- length(x)
fval <- 1.0 + sum (100 * (x[1:(n-1)]^2 - x[2:n])^2 + (x[2:n] - 1)^2)
return(fval)
}
@ 

This function has a global minimum at 1, which \pkg{DEoptim} finds for 
$n = 10$ with a
call like: 
<<ban1>>=
n <- 10
ans <- DEoptim(fn=genrose.f, lower=rep(-5, n), upper=rep(5, n),
               control=list(NP=100, itermax=4000,trace=FALSE))
@ 

The minimum can be determined with far fewer function evalutations 
with a gradient-based method such as ``BFGS'' \citep{nash}, e.g., with the 
call
<<ban2>>=
ans1 <- optim(par=runif(10,-5,5), fn=genrose.f, method="BFGS",
              control=list(maxit=4000))
              
@ 
Note further that users interested in exact reproduction of results should 
set the seed of their random number generator before calling \pkg{DEoptim}.
DE is a randomized algorithm, and the results may vary between runs.

\section{The Differential Evolution algorithm}\label{alg} 

We sketch the classical DE algorithm here and refer
interested readers to the work of \citet{price} and
\citet{Price:Storn:Lampinen:06} for further elaboration.  
The algorithm is an
evolutionary technique which at each generation transforms a set of
parameter vectors, termed the population, into another set of parameter
vectors, the members of which are more likely to minimize the
objective function.  In order generate a new parameter vector, DE disturbs 
 an old parameter vector with the scaled difference of two randomly selected 
 parameter vectors.  

The variable $\mathit{NP}$ represents the number of
parameter vectors in the population.  At generation 0, $\mathit{NP}$ guesses
for the optimal value of the parameter vector are made, either using
random values between upper and lower bounds for each parameter or
using values given by the user.  
Each generation involves creation of a new population from the current
population members $x_{i,g}$, 
 where  $i$ indexes the
vectors that make up the population and 
$g$ indexes generation. 
This is
accomplished using {\sl differential mutation} of the population
members.  A trial mutant parameter vector $v_{i,g}$ is created by choosing
three members of the population, $x_{r0,g}$, $x_{r1,g}$ and
$x_{r2,g}$, at random.  Then $v_{i,g}$ is generated as
\begin{equation}
v_{i,g} \doteq x_{r0,g} + \text{F} \cdot (x_{r1,g} - x_{r2,g})
\label{de} 
\end{equation}
where $F$ is a positive scale factor.  Effective values of $\text{F}$ are 
typically less than 1. 

After the first mutation operation, mutation is continued until either
$\text{length}(x)$ mutations have been made or $\mathit{rand} > \mathit{CR}$,
where $\mathit{CR}$ 
is a crossover probability $\mathit{CR} \in [0,1]$, and where here and 
throughout $\mathit{rand}$ is used to denote a random number from 
$\mathcal{U}(0,1)$. The crossover probability $\mathit{CR}$
controls the fraction of the parameter values that are copied from the mutant.
$\mathit{CR}$ approximates but does not 
exactly represent the probability that a parameter value will be inherited 
from the mutant, since at least one mutation always occurs.  Mutation is applied
in this way to each member of the population.  

If an element $v_j$ of the parameter vector is found to violate the bounds
after mutation and crossover, it is reset, where here and throughout we use
$j$ to index into a parameter vector.
In the implementation of \pkg{DEoptim}, if $v_{j} >
\text{upper}_j$, it is reset as $v_j \doteq \text{upper}_j - \mathit{rand}
\cdot ( \text{upper}_j - \text{lower}_j)$, and if $v_{j} <
\text{lower}_j$, it is reset as $v_j \doteq \text{lower}_j + \mathit{rand}
\cdot ( \text{upper}_j - \text{lower}_j)$.  This ensures that candidate 
population members found to violate the bounds are set some random amount 
away from them, in such a way that the bounds are guaranteed to be satisfied.
Then the objective function
values associated with the children $v$ are determined.  If a trial
vector $v_{i,g}$ has equal or lower objective function value than the
vector $x_{i,g}$, $v_{i,g}$ replaces $x_{i,g}$ in the population;
otherwise $x_{i,g}$ remains.

The algorithm stops after some set number of generations, or after the objective 
function value associated with the best member has been reduced below some set 
threshold, or if it is unable to reduce the best member by a certain value over
over set number of iterations.

Variations on this theme are possible,
some of which are described in the following section.   
Values of  $\mathit{NP}$ and  $\mathit{CR}$ that have been 
found to be most effective for a variety of problems are described in 
\citet[Section~2]{Price:Storn:Lampinen:06}. Reasonable default values for many problems
are given in the following section. 

\section{Implementation}\label{imp} 

\pkg{DEoptim} was first published on the Comprehensive \proglang{R} Archive
Network (CRAN) in 2005 by David Ardia.   
Early versions were written in pure  \proglang{R}.
Since version 2.0-0 (published to CRAN in 2009 by Katharine Mullen) 
the package has relied on an
interface to  a \proglang{C} implementation of DE, which is significantly 
faster on most problems as compared to the implementation in pure \proglang{R}.
 Since version 2.0-3 the \proglang{C}
implementation dynamically allocates the memory required to store the
population, removing limitations on the number of members in the
population and length of the parameter vectors that may be optimized.

The implementation is used by calling the \proglang{R}
function \code{DEoptim}, the arguments of which are:
\begin{itemize}
\item \code{fn}: The objective function to be minimized.  This function 
  should have as its first argument the vector of real-valued parameters to optimize, and 
  return a scalar real result.  
\item \code{lower}, \code{upper}: Vectors specifying scalar real lower and upper 
  bounds on each parameter to be optimized, so that the $i$th element of
  \code{lower} and \code{upper} applies to the $i$th parameter.  The implementation
  searches between \code{lower} and \code{upper} for the global optimum of \code{fn}.
\item \code{control}: A list of control parameters, discussed below. 
  \item \code{...}: allows the user to pass additional arguments to the function \code{fn}.
\end{itemize}

The \code{control} argument is a list, 
the following elements of which are currently interpreted:
\begin{itemize}
\item \code{VTR}: The value to reach.  Specify the global minimum of
  \code{fn} if it is known, or if you wish to cease optimization after
  having reached a certain value.  The default value is \code{-Inf}.
\item \code{strategy}:  This defines the Differential Evolution
  strategy used in the optimization procedure, described below in the 
  terms used by \citet{Price:Storn:Lampinen:06}: 
  \begin{itemize}
  \item \code{1}: DE / rand / 1 / bin (classical strategy). 
    This strategy is the classical approach described in Section~\ref{alg}. 
  \item \code{2}: DE / local-to-best / 1 / bin. 
    In place of the classical DE mutation given in~\eqref{de}, 
    the expression
    \begin{align*}
      v_{i,g} \doteq \text{old}_{i,g} + \text{F} \cdot (\text{best}_{g} - \text{old}_{i,g}) + \text{F} \cdot (x_{r1,g} - x_{r2,g}) 
    \end{align*}
     is used, where  $\text{old}_{i,g}$ and $\text{best}_{g}$  are the 
     $i$th member and best member, respectively, of the previous population.
     This strategy is currently used by default.  
     
   \item \code{3}: DE / best / 1 / bin with jitter.
     In place of the classical DE mutation given in~\eqref{de}, 
     the expression
     \begin{align*}
       v_{i,g} \doteq \text{best}_{g} + \text{jitter} + \text{F} \cdot (x_{r1,g} - x_{r2,g}) 
     \end{align*}
     is used, where $\text{jitter}$ is defined as 
     $0.0001 \cdot \mathit{rand} + F$.
      \item \code{4}: DE / rand / 1 / bin with per vector dither.
     In place of the classical DE mutation given in~\eqref{de}, 
     the expression
     \begin{align*}
     v_{i,g} \doteq x_{r0,g} + \text{dither} \cdot (x_{r1,g} - x_{r2,g})
     \end{align*}
     is used, where $\text{dither}$ is calculated as 
     $\text{dither} \doteq F + \mathit{rand} \cdot (1 - F)$.
      \item \code{5}: DE / rand / 1 / bin with per generation dither.
     The strategy described for \code{4} is used, but $\text{dither}$
     is only determined once per-generation. 
      \item \code{6}: DE / current-to-p-best / 1.
     The top $\text(100*p)$ percent best solutions are used in the mutation,
     where $p$ is defined in $\text(0,1]$.
   \item any value not above: variation to DE / rand / 1 / bin: either-or 
     algorithm.
     In the case that $\mathit{rand} < 0.5$, 
     the classical strategy described for \code{1} is used.  Otherwise, the 
     expression
     \begin{align*}
       v_{i,g} \doteq  x_{r0,g} + 0.5 \cdot (F + 1.0) \cdot (x_{r1,g} + x_{r2,g} -  2 \cdot x_{r0,g})
     \end{align*}
     is used. 
  \end{itemize}
\item  \code{bs}: 
  If \code{FALSE} then every mutant will be tested against a
  member in the previous generation, and the best value will survive 
  into the next generation.  This is the standard trial vs. target
  selection described in Section~\ref{alg}.   If \code{TRUE} 
  then the old generation and \code{NP}
  mutants will be sorted by their associated objective function
  values, and the best \code{NP} vectors will proceed into the next
  generation (this is best-of-parent-and-child selection).
  The default value is \code{FALSE}.
\item \code{NP}:  Number of population members. The default value is \code{50}.
\item \code{itermax}: The maximum iteration (population generation) allowed.
  The default value is \code{200}.
\item \code{CR}: Crossover probability from interval [0,1]. The default
  value is \code{0.5}.
\item \code{F}: Stepsize from interval [0,2]. The default value is \code{0.8}.
\item \code{trace}: Positive integer or logical value indicating whether 
  printing of progress occurs at each iteration. The default value is
  \code{TRUE}.  If a positive integer is specified, printing occurs every
  \code{trace} iterations.  
\item \code{initialpop}: An initial population used as a starting
  population in the optimization procedure, specified as a matrix in which each row represents
  a population member.   May be useful to speed up
  convergence. Defaults to \code{NULL}, so  that 
  the initial population is generated randomly within 
  the lower and upper boundaries.
\item \code{storepopfrom}: From which generation should the following
  intermediate populations be stored in memory. Default to
  \code{itermax + 1}, i.e., no intermediate population is stored.
\item \code{storepopfreq}: The frequency with which populations
  are stored. The default value is \code{1}, i.e., every intermediate population
  is stored.
   \item \code{checkWinner}: Logical value indicating whether to re-evaluate
      the objective function using the winning parameter vector if this
      vector remains the same between
    generations.  This may be useful for the optimization of a noisy
    objective function.  If \code{checkWinner = TRUE} and 
    \code{avWinner = FALSE}
   then the  value associated with re-evaluation of
   the objective function is used in the next generation.
   Default to \code{FALSE}.
 \item \code{avWinner}: Logical value.  If  \code{checkWinner = TRUE} and
   \code{avWinner = TRUE} then the objective
   function value associated with the winning member represents the
   average of all evaluations of the objective function over the course
   of the `winning streak' of the best population member. This option
   may be useful for optimization of noisy objective functions, and is
   interpreted only if \code{checkWinner = TRUE}. 
   The default value is \code{TRUE}.
 \end{itemize} 
The default value of \code{control} is the return value of 
\code{DEoptim.control()}, which is  a list  with the above elements and 
specified default values.  

The return value of the \code{DEoptim} function is a member of the S3 class 
\code{DEoptim}.  Members of this class have a \code{plot} method that 
accepts the argument \code{plot.type}.  
When \code{retVal} is an object returned by 
\pkg{DEoptim}, calling \code{plot(retVal, plot.type = "bestmemit")} results in
 a plot of the parameter values that represent the lowest value of the 
 objective function each generation.  
 Calling \code{plot(retVal, plot.type = "bestvalit")} plots the best value 
 of the objective function each generation.  Calling 
  \code{plot(retVal, plot.type = "storepop")} results in a plot of stored 
  populations (which are only available if these have been saved by 
  setting the \code{control} argument of \code{DEoptim} appropriately).  
A summary method for objects of S3 class \code{DEoptim} also exists, 
and returns the best parameter vector, the best value of the objective function,
the number of generations optimization ran, and the number of times the 
objective function was evaluated. 

A note on recommended settings: We have set the default values
to the methods recommended by \citet{Price:Storn:Lampinen:06} as starting 
points.  We use \code{strategy = 2} by default; the user should 
consider trying as alternatives \code{strategy = 6} and  \code{strategy = 1}, 
though the best method will be highly problem-dependent.
Generally, the user should set the lower and upper bounds to 
exploit the full allowable numerical range, i.e., if a parameter 
is allowed to exhibit
values in the range [-1, 1] it is typically 
a good idea to pick the initial values from this
range instead of unnecessarily restricting diversity. 
Increasing the value for \code{NP} will mean greater likelihood of finding the
minimum, but run-time will be longer.  

\pkg{DEoptim} relies on repeated evaluation of the objective function
in order to move the population toward a global minimum.  Users
interested in making \pkg{DEoptim} run as fast as possible should
ensure that evaluation of the objective function is as efficient as
possible.  Using pure \proglang{R} code, this may often be accomplished
using vectorization.  Writing parts of the objective function in a
lower-level language like \proglang{C} or \proglang{Fortran}
may also increase speed.

\section*{Disclaimer}

The views expressed in this vignette are the sole responsibility of the
authors and do not necessarily reflect those of NIST and 
aeris CAPITAL AG.

\bibliography{DEoptim}

\end{document}

\name{DEoptim.control}
\alias{DEoptim.control}
\title{Control various aspects of the DEoptim implementation}
\description{
  Allow the user to set some characteristics of the 
  Differential Evolution optimization algorithm implemented
  in \code{DEoptim}.
}
\usage{
DEoptim.control(VTR = -Inf, strategy = 2, bs = FALSE, NP = NA,
  itermax = 200, CR = 0.5, F = 0.8, trace = TRUE, initialpop = NULL,
  storepopfrom = itermax + 1, storepopfreq = 1, p = 0.2, c = 0, reltol,
  steptol, parallelType = 0, cluster = NULL, packages = c(), parVar = c(),
  foreachArgs = list())
}
\arguments{
   \item{VTR}{the value to be reached. The optimization process
      will stop if either the maximum number of iterations \code{itermax}
      is reached or the best parameter vector \code{bestmem} has found a value
      \code{fn(bestmem) <= VTR}. Default to \code{-Inf}.}
  \item{strategy}{defines the Differential Evolution
    strategy used in the optimization procedure:\cr
     \code{1}: DE / rand / 1 / bin (classical strategy)\cr
     \code{2}: DE / local-to-best / 1 / bin (default)\cr
     \code{3}: DE / best / 1 / bin with jitter\cr
     \code{4}: DE / rand / 1 / bin with per-vector-dither\cr
     \code{5}: DE / rand / 1 / bin with per-generation-dither\cr
     \code{6}: DE / current-to-p-best / 1\cr
     any value not above: variation to DE / rand / 1 / bin: either-or-algorithm.  Default
     strategy is currently \code{2}. See *Details*.
    }
    \item{bs}{if \code{FALSE} then every mutant will be tested against a
      member in the previous generation, and the best value will proceed
      into the next generation (this is standard trial vs. target
      selection). If \code{TRUE} then the old generation and \code{NP}
      mutants will be sorted by their associated objective function
      values, and the best \code{NP} vectors will proceed into the next
      generation (best of parent and child selection). Default is
    \code{FALSE}.}
    \item{NP}{number of population members. Defaults to \code{NA}; if
      the user does not change the value of \code{NP} from \code{NA} or
      specifies a value less than 4 it
      is reset when \code{DEoptim} is called as \code{10*length(lower)}. For
      many problems it is best to set
      \code{NP} to be at least 10 times the length
      of the parameter vector.  }
    \item{itermax}{the maximum iteration (population generation) allowed.
      Default is \code{200}.}
    \item{CR}{crossover probability from interval [0,1]. Default
      to \code{0.5}.}
    \item{F}{differential weighting factor from interval [0,2]. Default
      to \code{0.8}.}
    \item{trace}{Positive integer or logical value indicating whether 
  printing of progress occurs at each iteration. The default value is
  \code{TRUE}.  If a positive integer is specified, printing occurs every
  \code{trace} iterations.  }
    \item{initialpop}{an initial population used as a starting
      population in the optimization procedure. May be useful to speed up
      the convergence. Default to \code{NULL}.  If given, each member of
    the initial population should be given as a row of a numeric matrix, so that
   \code{initialpop} is a matrix with \code{NP} rows and a number of
   columns equal to the length of the parameter vector to be optimized. }
    \item{storepopfrom}{from which generation should the following
      intermediate populations be stored in memory. Default to
      \code{itermax + 1}, i.e., no intermediate population is stored.}
    \item{storepopfreq}{the frequency with which populations are stored.
      Default to \code{1}, i.e., every intermediate population
      is stored.}
 \item{p}{when \code{strategy = 6}, the top (100 * p)\% best 
   solutions are used in the mutation. \code{p} must be defined in (0,1].}
 \item{c}{\code{c} controls the speed of the
   crossover adaptation. Higher values of \code{c} give more weight to the
   current successful mutations. \code{c} must be defined in (0,1].}
 \item{reltol}{relative convergence tolerance.  The algorithm stops if
   it is unable to reduce the value by a factor of \code{reltol * (abs(val) +
   reltol)} after \code{steptol} steps. Defaults to
   \code{sqrt(.Machine$double.eps)}, typically about \code{1e-8}.}
 \item{steptol}{see \code{reltol}. Defaults to \code{itermax}.}
 \item{parallelType}{Defines the type of parallelization to employ, if
   any.
    \code{0}: The default, this uses \code{DEoptim} on one only one core.
    \code{1}: This uses all available cores, via the \pkg{parallel}
    package, to run \code{DEoptim}.  If \code{parallelType=1}, then the
    \code{packages} argument and the \code{parVar} argument need to
    specify the packages required by the objective function and the
    variables required in the environment, respectively. 
    \code{2}: This uses the \pkg{foreach} package for parallelism; see
    the \code{sandbox} directory in the source code for examples.  If
    \code{parallelType=2}, then the \code{foreachArgs} argument can pass
    the options to be called with \code{foreach}.  
  }
   \item{cluster}{Existing \pkg{parallel} cluster object. If provided, overrides
+  specified \code{parallelType}. Using \code{cluster} allows fine-grained control
+  over the number of used cores and exported data.}
  \item{packages}{Used if  \code{parallelType=1}; a list of
    package names (as strings) that need to be loaded for use by the objective
  function. }
\item{parVar}{Used if  \code{parallelType=1};  a list of variable names
  (as strings) that need to exist in the environment for use by the
  objective function or are used as arguments by the objective
  function. }  
  \item{foreachArgs}{A list of named arguments for the \code{foreach}
    function from the
    package \pkg{foreach}.  The arguments \code{i}, \code{.combine} and
    \code{.export} are not possible to set here; they are set
    internally. }
 }
\value{
  The default value of \code{control} is the return value of 
  \code{DEoptim.control()}, which is  a list (and a member of the
  \code{S3} class  
  \code{DEoptim.control}) with the above elements.
}
\details{
  This defines the Differential Evolution 
  strategy used in the optimization procedure, described below in the 
  terms used by Price et al. (2006); see also Mullen et al. (2009) for details. 
  
  \itemize{
    \item \code{strategy = 1}: DE / rand / 1 / bin. \cr 
    This strategy is the classical approach for DE, and is described in \code{\link{DEoptim}}. 
  
    \item \code{strategy = 2}: DE / local-to-best / 1 / bin. \cr
    In place of the classical DE mutation the expression
    \deqn{
      v_{i,g} = old_{i,g} + (best_{g} - old_{i,g}) + x_{r0,g} + F \cdot (x_{r1,g} - x_{r2,g})
    }{
      v_i,g = old_i,g + (best_g - old_i,g) + x_r0,g + F * (x_r1,g - x_r2,g)
    } 
    is used, where \eqn{old_{i,g}}{old_i,g} and \eqn{best_{g}}{best_g} are the 
    \eqn{i}-th member and best member, respectively, of the previous population.
    This strategy is currently used by default.  
  
    \item \code{strategy = 3}: DE / best / 1 / bin with jitter.\cr
    In place of the classical DE mutation the expression
    \deqn{
       v_{i,g} = best_{g} + jitter + F \cdot (x_{r1,g} - x_{r2,g}) 
     }{
       v_i,g = best_g + jitter + F * (x_r1,g - x_r2,g) 
     }
     is used, where \eqn{jitter} is defined as 0.0001 * \code{rand} + F.
  
    \item \code{strategy = 4}: DE / rand / 1 / bin with per vector dither.\cr
     In place of the classical DE mutation the expression
    \deqn{
       v_{i,g} = x_{r0,g} + dither \cdot (x_{r1,g} - x_{r2,g})
     }{
       v_i,g = x_r0,g + dither * (x_r1,g - x_r2,g)
     }
     is used, where \eqn{dither} is calculated as \eqn{F + \code{rand} * (1 - F)}.
      
     \item \code{strategy = 5}: DE / rand / 1 / bin with per generation dither.\cr
     The strategy described for \code{4} is used, but \eqn{dither}
     is only determined once per-generation. 
     
     \item \code{strategy = 6}: DE / current-to-p-best / 1.\cr
     The top \eqn{(100*p)} percent best solutions are used in the mutation,
     where \eqn{p} is defined in \eqn{(0,1]}.
     
     \item any value not above: variation to DE / rand / 1 / bin: either-or algorithm.\cr
     In the case that \code{rand} < 0.5, the classical strategy \code{strategy = 1} is used. 
     Otherwise, the expression
     \deqn{
       v_{i,g} = x_{r0,g} + 0.5 \cdot (F + 1) \cdot (x_{r1,g} + x_{r2,g} -  2 \cdot x_{r0,g})
     }{
       v_i,g = x_r0,g + 0.5 * (F + 1) * (x_r1,g + x_r2,g -  2 * x_r0,g)
     }
     is used. 
  }
  
  Several conditions can cause the optimization process to stop:
  \itemize{
    \item{if the best parameter vector (\code{bestmem}) produces a value
      less than or equal to \code{VTR} (i.e. \code{fn(bestmem) <= VTR}), or}
    \item{if the maximum number of iterations is reached (\code{itermax}), or}
    \item{if a number (\code{steptol}) of consecutive iterations are unable
      to reduce the best function value by a certain amount (\code{reltol *
      (abs(val) + reltol)}). \code{100*reltol} is approximately the percent
      change of the objective value required to consider the parameter set
      an improvement over the current best member.}
  }

  Zhang and Sanderson (2009) define several extensions to the DE algorithm, 
  including strategy 6, DE/current-to-p-best/1. They also define a self-adaptive 
  mechanism for the other control parameters.  This self-adaptation will speed 
  convergence on many problems, and is defined by the control parameter \code{c}.
  If \code{c} is non-zero, crossover and mutation will be adapted by the algorithm.  
  Values in the range of \code{c=.05} to \code{c=.5} appear to work best for most 
  problems, though the adaptive algorithm is robust to a wide range of \code{c}.
    
}
\note{
  Further details and examples of the \R package \pkg{DEoptim} can be found
  in Mullen et al. (2011) and Ardia et al. (2011a, 2011b) or look at the 
  package's vignette by typing \code{vignette("DEoptim")}. Also, an illustration of
  the package usage for a high-dimensional non-linear portfolio optimization problem 
  is available by typing \code{vignette("DEoptimPortfolioOptimization")}. 

  Please cite the package in publications. Use \code{citation("DEoptim")}.
}
\seealso{
  \code{\link{DEoptim}} and \code{\link{DEoptim-methods}}.
}
\references{
  Ardia, D., Boudt, K., Carl, P., Mullen, K.M., Peterson, B.G. (2011)
  Differential Evolution with \pkg{DEoptim}. An Application to Non-Convex Portfolio Optimization. 
  \emph{R Journal}, 3(1), 27-34. 
  \doi{10.32614/RJ-2011-005}

  Ardia, D., Ospina Arango, J.D., Giraldo Gomez, N.D. (2011)
  Jump-Diffusion Calibration using Differential Evolution. 
  \emph{Wilmott Magazine}, 55 (September), 76-79.
  \doi{10.1002/wilm.10034}

  Mullen, K.M, Ardia, D., Gil, D., Windover, D., Cline,J. (2011). 
  \pkg{DEoptim:} An R Package for Global Optimization by Differential Evolution. 
  \emph{Journal of Statistical Software}, 40(6), 1-26.
  \doi{10.18637/jss.v040.i06}
  
  Price, K.V., Storn, R.M., Lampinen J.A. (2006)
  \emph{Differential Evolution - A Practical Approach to Global Optimization}.
  Berlin Heidelberg: Springer-Verlag. ISBN 3540209506.
  
  Zhang, J. and Sanderson, A. (2009) 
  \emph{Adaptive Differential Evolution}
  Springer-Verlag. ISBN 978-3-642-01526-7
}
\author{
  David Ardia, Katharine Mullen \email{mullenkate@gmail.com}, 
  Brian Peterson and Joshua Ulrich.
}
\examples{
## set the population size to 20
DEoptim.control(NP = 20)

## set the population size, the number of iterations and don't
## display the iterations during optimization
DEoptim.control(NP = 20, itermax = 100, trace = FALSE)
   
}
\keyword{nonlinear}
\keyword{optimize}

\name{DEoptim-methods}
\alias{DEoptim-methods}
\alias{plot.DEoptim}
\alias{summary.DEoptim}
\title{DEoptim-methods}
\description{Methods for DEoptim objects.}
\usage{
\method{summary}{DEoptim}(object, \dots)
\method{plot}{DEoptim}(x, plot.type = c("bestmemit", "bestvalit", "storepop"), \dots)
}
\arguments{
  \item{object}{an object of class \code{DEoptim}; usually, a result
    of a call to \code{\link{DEoptim}}.}
  \item{x}{an object of class \code{DEoptim}; usually, a result
    of a call to \code{\link{DEoptim}}.}
  \item{plot.type}{should we plot the best member at each iteration, the best value
    at each iteration or the intermediate populations?}
  \item{\dots}{further arguments passed to or from other methods.}
}
\details{
  Members of the class \code{DEoptim} have a \code{plot} method that
  accepts the argument \code{plot.type}. \cr
  \code{plot.type = "bestmemit"} results
  in a plot of the parameter values that represent the lowest value of the objective function
  each generation. \code{plot.type = "bestvalit"} plots the best value of
  the objective function each generation. Finally, \code{plot.type = "storepop"} results in a plot of
  stored populations (which are only available if these have been saved by
  setting the \code{control} argument of \code{DEoptim} appropriately). Storing intermediate populations 
  allows us to examine the progress of the optimization in detail.   
  A summary method also exists and returns the best parameter vector, the best value of the objective function,
  the number of generations optimization ran, and the number of times the 
  objective function was evaluated. 
}
\note{
  Further details and examples of the \R package \pkg{DEoptim} can be found
  in Mullen et al. (2011) and Ardia et al. (2011a, 2011b) or look at the 
  package's vignette by typing \code{vignette("DEoptim")}.

  Please cite the package in publications. Use \code{citation("DEoptim")}.
}
\author{
  David Ardia, Katharine Mullen \email{mullenkate@gmail.com}, 
  Brian Peterson and Joshua Ulrich.
}
\seealso{
  \code{\link{DEoptim}} and \code{\link{DEoptim.control}}.
}
\references{
  Ardia, D., Boudt, K., Carl, P., Mullen, K.M., Peterson, B.G. (2011)
  Differential Evolution with \pkg{DEoptim}. An Application to Non-Convex Portfolio Optimization. 
  \emph{R Journal}, 3(1), 27-34. 
  \doi{10.32614/RJ-2011-005}

  Ardia, D., Ospina Arango, J.D., Giraldo Gomez, N.D. (2011)
  Jump-Diffusion Calibration using Differential Evolution. 
  \emph{Wilmott Magazine}, 55 (September), 76-79.
  \doi{10.1002/wilm.10034}

  Mullen, K.M, Ardia, D., Gil, D., Windover, D., Cline,J. (2011). 
  \pkg{DEoptim:} An R Package for Global Optimization by Differential Evolution. 
  \emph{Journal of Statistical Software}, 40(6), 1-26.
  \doi{10.18637/jss.v040.i06}
}
\examples{
  ## Rosenbrock Banana function
  ## The function has a global minimum f(x) = 0 at the point (1,1).  
  ## Note that the vector of parameters to be optimized must be the first 
  ## argument of the objective function passed to DEoptim.
  Rosenbrock <- function(x){
    x1 <- x[1]
    x2 <- x[2]
    100 * (x2 - x1 * x1)^2 + (1 - x1)^2
  }

  lower <- c(-10, -10)
  upper <- -lower
  
  set.seed(1234)
  outDEoptim <- DEoptim(Rosenbrock, lower, upper)
  
  ## print output information
  summary(outDEoptim)

  ## plot the best members
  plot(outDEoptim, type = 'b')

  ## plot the best values
  dev.new()
  plot(outDEoptim, plot.type = "bestvalit", type = 'b', col = 'blue')

  ## rerun the optimization, and store intermediate populations
  outDEoptim <- DEoptim(Rosenbrock, lower, upper,
                        DEoptim.control(itermax = 500,
                        storepopfrom = 1, storepopfreq = 2))
  summary(outDEoptim)
  
  ## plot intermediate populations
  dev.new()
  plot(outDEoptim, plot.type = "storepop")

  ## Wild function
  Wild <- function(x)
    10 * sin(0.3 * x) * sin(1.3 * x^2) +
       0.00001 * x^4 + 0.2 * x + 80

  outDEoptim = DEoptim(Wild, lower = -50, upper = 50,
                       DEoptim.control(trace = FALSE, storepopfrom = 50,
                       storepopfreq = 1))
  
  plot(outDEoptim, type = 'b')

  dev.new()
  plot(outDEoptim, plot.type = "bestvalit", type = 'b')

\dontrun{
  ## an example with a normal mixture model: requires package mvtnorm
  library(mvtnorm)

  ## neg value of the density function
  negPdfMix <- function(x) {
     tmp <- 0.5 * dmvnorm(x, c(-3, -3)) + 0.5 * dmvnorm(x, c(3, 3))
     -tmp
  }

  ## wrapper plotting function
  plotNegPdfMix <- function(x1, x2)
     negPdfMix(cbind(x1, x2))

  ## contour plot of the mixture
  x1 <- x2 <- seq(from = -10.0, to = 10.0, by = 0.1)
  thexlim <- theylim <- range(x1)
  z <- outer(x1, x2, FUN = plotNegPdfMix)
  
  contour(x1, x2, z, nlevel = 20, las = 1, col = rainbow(20),
     xlim = thexlim, ylim = theylim)

  set.seed(1234)
  outDEoptim <- DEoptim(negPdfMix, c(-10, -10), c(10, 10),
     DEoptim.control(NP = 100, itermax = 100, storepopfrom = 1,
     storepopfreq = 5))

  ## convergence plot
  dev.new()
  plot(outDEoptim)
  
  ## the intermediate populations indicate the bi-modality of the function
  dev.new()
  plot(outDEoptim, plot.type = "storepop")
}
}
\keyword{methods}

\name{SMI}
\alias{SMI}
\alias{y}
\docType{data}
\title{Swiss Market Index data}
\usage{
data("SMI")
}
\description{
  See Mullen et al. (2011) for description of this dataset.
}
\references{
  Mullen, K.M, Ardia, D., Gil, D., Windover, D., Cline,
  J. (2011). \emph{\pkg{DEoptim:} An R Package for Global Optimization by
  Differential Evolution}. Journal of Statistical Software, 40(6), 1-26.
  URL \url{http://www.jstatsoft.org/v40/i06/}.
}
\keyword{datasets}

\name{DEoptim}
\alias{DEoptim}
\title{Differential Evolution Optimization}
\concept{minimization}
\description{
  Performs evolutionary global optimization via the Differential Evolution algorithm.
}
\usage{
DEoptim(fn, lower, upper, control = DEoptim.control(), ..., fnMap=NULL)
}
\arguments{
  \item{fn}{the function to be optimized (minimized). The function should have as its first 
    argument the vector of real-valued parameters to optimize, and return a scalar real result. \code{NA} 
    and \code{NaN} values are not allowed.}
  \item{lower, upper}{two vectors specifying scalar real lower and upper
    bounds on each parameter to be optimized, so that the i-th element 
    of \code{lower} and \code{upper} applies to the i-th parameter. The implementation searches
    between \code{lower} and \code{upper} for the global optimum (minimum) of \code{fn}.}
  \item{control}{a list of control parameters; see \code{\link{DEoptim.control}}.}
  \item{fnMap}{an optional function that will be run after each population is
    created, but before the population is passed to the objective function. This
    allows the user to impose integer/cardinality constriants.}
  \item{...}{further arguments to be passed to \code{fn}.}
}
\details{
  \code{DEoptim} performs optimization (minimization) of \code{fn}. 

  The \code{control} argument is a list; see the help file for
  \code{\link{DEoptim.control}} for details.  

  The \R implementation of Differential Evolution (DE), \pkg{DEoptim}, was first published on the Comprehensive \R Archive
  Network (CRAN) in 2005 by David Ardia. Early versions were written in 
  pure \R. Since version 2.0-0 (published to CRAN in 2009) the package has relied on an
  interface to a C implementation of DE, which is significantly 
  faster on most problems as compared to the implementation in 
  pure \R. The C interface is in many respects similar to the MS
  Visual C++ v5.0 implementation of the Differential Evolution algorithm
  distributed with the book \emph{Differential Evolution -- A Practical
  Approach to Global Optimization} by Price, K.V., Storn, R.M., Lampinen
  J.A, Springer-Verlag, 2006. Since version
  2.0-3 the C implementation dynamically allocates the memory required
  to store the population, removing limitations on the number of members
  in the population and length of the parameter vectors that may be
  optimized.  Since version 2.2-0, the package allows for parallel
  operation, so that the evaluations of the objective function may be 
  performed using all available cores.  This is accomplished using either
  the built-in \pkg{parallel} package or the \pkg{foreach} package.  If
  parallel operation is desired, the user should set \code{parallelType}
  and make sure that the arguments and packages needed by the objective
  function are available; see \code{\link{DEoptim.control}}, the example
  below and examples in the sandbox directory for details.  
  
  Since becoming publicly available, the package \pkg{DEoptim} has been
  used by several authors to solve optimization problems arising in
  diverse domains; see Mullen et al. (2011) for a review.
  
  To perform a maximization (instead of minimization) of a given
  function, simply define a new function which is the opposite of the
  function to maximize and apply \code{DEoptim} to it.
  
  To integrate additional constraints (other than box constraints) on the parameters \code{x} of
  \code{fn(x)}, for instance \code{x[1] + x[2]^2 < 2}, integrate the
  constraint within the function to optimize, for instance: 
  \preformatted{
    fn <- function(x)\{
      if (x[1] + x[2]^2 >= 2)\{
        r <- Inf
      else\{
        ...
      \}
      return(r)
    \}
  }
  This simplistic strategy usually does not work all that well for gradient-based or Newton-type 
  methods. It is likely to be alright when the solution is in the interior of the feasible region, but when 
  the solution is on the boundary, optimization algorithm would have a difficult time converging. Furthermore, when
  the solution is on the boundary, this strategy would make the algorithm converge to an inferior solution in the interior. 
  However, for methods such as DE which are not gradient based, this strategy might not be that bad.

  Note that \code{DEoptim} stops if any \code{NA} or \code{NaN} value is
  obtained. You have to redefine your function to handle these values
  (for instance, set \code{NA} to \code{Inf} in your objective function).

  It is important to emphasize that the result of \code{DEoptim} is a random variable, 
  i.e., different results may be obtained when the algorithm is run repeatedly with the same 
  settings. Hence, the user should set the random seed if they want to reproduce the results, e.g., by 
  setting \code{set.seed(1234)} before the call of \code{DEoptim}.

  \code{DEoptim} relies on repeated evaluation of the objective function
  in order to move the population toward a global minimum. Users
  interested in making \code{DEoptim} run as fast as possible should
  consider using the package in parallel mode (so that all CPU's
  available are used), and also 
  ensure that evaluation of the objective function is as efficient as
  possible (e.g. by using vectorization in pure \R code, or
  writing parts of the objective function in a
  lower-level language like C or Fortran).
  
  Further details and examples of the \R package \pkg{DEoptim} can be found
  in Mullen et al. (2011) and Ardia et al. (2011a, 2011b) or look at the 
  package's vignette by typing \code{vignette("DEoptim")}. Also, an illustration of
  the package usage for a high-dimensional non-linear portfolio optimization problem 
  is available by typing \code{vignette("DEoptimPortfolioOptimization")}. 
  
  Please cite the package in publications. Use \code{citation("DEoptim")}.
}
\value{
  The output of the function \code{DEoptim} is a member of the \code{S3} class \code{DEoptim}. More precisely,
  this is a list (of length 2) containing the following elements:\cr

  \code{optim}, a list containing the following elements:
  \itemize{
    \item \code{bestmem}: the best set of parameters found.
    \item \code{bestval}: the value of \code{fn} corresponding to \code{bestmem}.
    \item \code{nfeval}: number of function evaluations.
    \item \code{iter}: number of procedure iterations.
  }

  \code{member}, a list containing the following elements:
  \itemize{
    \item \code{lower}: the lower boundary.
    \item \code{upper}: the upper boundary.
    \item \code{bestvalit}: the best value of \code{fn} at each iteration.
    \item \code{bestmemit}: the best member at each iteration.
    \item \code{pop}: the population generated at the last iteration.
    \item \code{storepop}: a list containing the intermediate populations.
  }

  Members of the class \code{DEoptim} have a \code{plot} method that
  accepts the argument \code{plot.type}.\cr
  \code{plot.type = "bestmemit"} results
  in a plot of the parameter values that represent the lowest value of the objective function
  each generation. \code{plot.type = "bestvalit"} plots the best value of
  the objective function each generation. Finally,
  \code{plot.type = "storepop"} results in a plot of
  stored populations (which are only available if these have been saved by
  setting the \code{control} argument of \code{DEoptim} appropriately). Storing intermediate populations 
  allows us to examine the progress of the optimization in detail.   
  A summary method also exists and returns the best parameter vector, the best value of the objective function,
  the number of generations optimization ran, and the number of times the 
  objective function was evaluated. 
}
\note{
  \emph{Differential Evolution} (DE) is a search heuristic introduced by Storn and Price (1997). 
  Its remarkable performance as a global optimization algorithm on continuous numerical minimization 
  problems has been extensively explored; see Price et al. (2006). DE belongs to the class of genetic 
  algorithms which use biology-inspired operations of 
  crossover, mutation, and selection on a population in order to minimize an objective 
  function over the course of successive generations (see Mitchell, 1998). As with other evolutionary algorithms, 
  DE solves optimization problems by evolving a population of candidate solutions using alteration and selection
  operators. DE uses floating-point instead of bit-string encoding of population members, and 
  arithmetic operations instead of logical operations in mutation. DE is particularly well-suited to find the global optimum of a 
  real-valued function of real-valued parameters, and does not require that the function be
  either continuous or differentiable.

  Let \eqn{\mathit{NP}}{NP} denote the number of parameter vectors (members) \eqn{x \in R^d}{x in R^d} in the population. 
  In order to create the initial generation, \eqn{\mathit{NP}}{NP} guesses for the optimal value
  of the parameter vector are made, either using random values between lower and upper 
  bounds (defined by the user) or using values given by
  the user. Each generation involves creation of a new population from
  the current population members \eqn{\{ x_i \,|\, i = 1, \ldots, \mathit{NP}\}}{{x_i | i=1,...,NP}}, 
  where \eqn{i} indexes the vectors that make up the population.
  This is accomplished using \emph{differential mutation} of the
  population members. An initial mutant parameter vector \eqn{v_i} is
  created by choosing three members of the population, \eqn{x_{r_0}},
  \eqn{x_{r_1}} and \eqn{x_{r_2}}, at random. Then \eqn{v_i} is
  generated as

  \deqn{v_i \doteq x_{r_0} + \mathit{F} \cdot (x_{r_1} - x_{r_2})}{v_i := x_{r_0} + F * (x_{r_1} - x_{r_2})}  

  where \eqn{\mathit{F}}{F} is the differential weighting factor, effective values for which are
  typically between 0 and 1. After the first mutation operation, mutation is
  continued until \eqn{d} mutations have been made, with a crossover probability 
  \eqn{\mathit{CR} \in [0,1]}{CR in [0,1]}.
  The crossover probability \eqn{\mathit{CR}}{CR} controls the fraction of the parameter
  values that are copied from the mutant. If an element of the trial parameter vector is found to violate the
  bounds after mutation and crossover, it is reset in such a way that the bounds are respected (with the
  specific protocol depending on the implementation).
  Then, the objective function values associated with the children are determined. If a trial
  vector has equal or lower objective function value than the previous
  vector it replaces the previous vector in the population;
  otherwise the previous vector remains. Variations of this scheme have also
  been proposed; see Price et al. (2006) and \code{\link{DEoptim.control}}.

  Intuitively, the effect of the scheme is that the shape of the distribution of the population in the
  search space is converging with respect to size and direction towards areas with high
  fitness. The closer the population gets to the global optimum, the more the distribution
  will shrink and therefore reinforce the generation of smaller difference vectors.  

  As a general advice regarding the choice of \eqn{\mathit{NP}}{NP}, \eqn{\mathit{F}}{F} and \eqn{\mathit{CR}}{CR}, 
  Storn et al. (2006) state the following: Set the number
  of parents \eqn{\mathit{NP}}{NP} to 10 times the number of parameters, select differential weighting factor 
  \eqn{\mathit{F} = 0.8}{F = 0.8} and crossover constant \eqn{\mathit{CR} = 0.9}{CR = 0.9}. Make sure that you initialize your parameter vectors
  by exploiting their full numerical range, i.e., if a parameter is allowed to exhibit
  values in the range [-100, 100] it is a good idea to pick the initial values from this
  range instead of unnecessarily restricting diversity. If you experience misconvergence in 
  the optimization process you usually have to increase the value for \eqn{\mathit{NP}}{NP}, but often you only have to adjust
  \eqn{\mathit{F}}{F} to be a little lower or higher than 0.8. If you increase
  \eqn{\mathit{NP}}{NP} and simultaneously lower \eqn{\mathit{F}}{F} a little, convergence is more
  likely to occur but generally takes longer, i.e., DE is getting
  more robust (there is always a convergence speed/robustness trade-off).

  DE is much more sensitive to the choice of \eqn{\mathit{F}}{F} than it is to
  the choice of \eqn{\mathit{CR}}{CR}. \eqn{\mathit{CR}}{CR} is more like a fine tuning element. High
  values of \eqn{\mathit{CR}}{CR} like \eqn{\mathit{CR} = 1}{CR = 1} give faster convergence if convergence
  occurs. Sometimes, however, you have to go down as much as \eqn{\mathit{CR} = 0}{CR = 0} to
  make DE robust enough for a particular problem. For more details on the DE strategy, we refer 
  the reader to Storn and Price (1997) and Price et al. (2006).
}
\references{
  Ardia, D., Boudt, K., Carl, P., Mullen, K.M., Peterson, B.G. (2011)
  Differential Evolution with \pkg{DEoptim}. An Application to Non-Convex Portfolio Optimization. 
  \emph{R Journal}, 3(1), 27-34. 
  \doi{10.32614/RJ-2011-005}

  Ardia, D., Ospina Arango, J.D., Giraldo Gomez, N.D. (2011)
  Jump-Diffusion Calibration using Differential Evolution. 
  \emph{Wilmott Magazine}, 55 (September), 76-79.
  \doi{10.1002/wilm.10034}
  
  Mitchell, M. (1998) 
  \emph{An Introduction to Genetic Algorithms}.
  The MIT Press. ISBN 0262631857.

  Mullen, K.M, Ardia, D., Gil, D., Windover, D., Cline,J. (2011). 
  \pkg{DEoptim:} An R Package for Global Optimization by Differential Evolution. 
  \emph{Journal of Statistical Software}, 40(6), 1-26.
  \doi{10.18637/jss.v040.i06}

  Price, K.V., Storn, R.M., Lampinen J.A. (2006)
  \emph{Differential Evolution - A Practical Approach to Global Optimization}.
  Berlin Heidelberg: Springer-Verlag. ISBN 3540209506.

  Storn, R. and Price, K. (1997) 
  Differential Evolution -- A Simple and Efficient Heuristic for Global Optimization over Continuous Spaces,
  \emph{Journal of Global Optimization}, 11:4, 341--359.  
}
\author{
  David Ardia, Katharine Mullen \email{mullenkate@gmail.com}, 
  Brian Peterson and Joshua Ulrich.
}
\seealso{
  \code{\link{DEoptim.control}} for control arguments,
  \code{\link{DEoptim-methods}} for methods on \code{DEoptim} objects,
  including some examples in plotting the results;
  \code{\link{optim}} or \code{\link{constrOptim}}
  for alternative optimization algorithms.
}
\examples{
  ## Rosenbrock Banana function
  ## The function has a global minimum f(x) = 0 at the point (1,1).  
  ## Note that the vector of parameters to be optimized must be the first 
  ## argument of the objective function passed to DEoptim.
  Rosenbrock <- function(x){
    x1 <- x[1]
    x2 <- x[2]
    100 * (x2 - x1 * x1)^2 + (1 - x1)^2
  }

  ## DEoptim searches for minima of the objective function between
  ## lower and upper bounds on each parameter to be optimized. Therefore
  ## in the call to DEoptim we specify vectors that comprise the
  ## lower and upper bounds; these vectors are the same length as the
  ## parameter vector.
  lower <- c(-10,-10)
  upper <- -lower
 
  ## run DEoptim and set a seed first for replicability
  set.seed(1234)
  DEoptim(Rosenbrock, lower, upper)

  ## increase the population size
  DEoptim(Rosenbrock, lower, upper, DEoptim.control(NP = 100))

  ## change other settings and store the output
  outDEoptim <- DEoptim(Rosenbrock, lower, upper, DEoptim.control(NP = 80,
                        itermax = 400, F = 1.2, CR = 0.7))
  
  ## plot the output
  plot(outDEoptim)

  ## 'Wild' function, global minimum at about -15.81515
  Wild <- function(x)
    10 * sin(0.3 * x) * sin(1.3 * x^2) +
       0.00001 * x^4 + 0.2 * x + 80

  plot(Wild, -50, 50, n = 1000, main = "'Wild function'")

  outDEoptim <- DEoptim(Wild, lower = -50, upper = 50,
                        control = DEoptim.control(trace = FALSE))
  
  plot(outDEoptim)

  DEoptim(Wild, lower = -50, upper = 50,
          control = DEoptim.control(NP = 50))
 
  ## The below examples shows how the call to DEoptim can be
  ## parallelized.
  ## Note that if your objective function requires packages to be
  ## loaded or has arguments supplied via \code{...}, these should be
  ## specified using the \code{packages} and \code{parVar} arguments
  ## in control.  
  \dontrun{ 

  Genrose <- function(x) {
     ## One generalization of the Rosenbrock banana valley function (n parameters)
     n <- length(x)
     ## make it take some time ... 
     Sys.sleep(.001) 
     1.0 + sum (100 * (x[-n]^2 - x[-1])^2 + (x[-1] - 1)^2)
  }

  # get some run-time on simple problems
  maxIt <- 250                     
  n <- 5

  oneCore <- system.time( DEoptim(fn=Genrose, lower=rep(-25, n), upper=rep(25, n),
                   control=list(NP=10*n, itermax=maxIt)))

  withParallel <-  system.time( DEoptim(fn=Genrose, lower=rep(-25, n), upper=rep(25, n),
                   control=list(NP=10*n, itermax=maxIt, parallelType=1)))

  ## Compare timings 
  (oneCore)
  (withParallel)
 }
}
\keyword{nonlinear}
\keyword{optimize}

\name{xrrData}
\alias{xrrData}
\docType{data}
\title{X-ray reflectometry data}
\usage{
data("xrrData")
}
\description{
  See Mullen et al. (2011) for description of this dataset.
}
\references{
  Mullen, K.M, Ardia, D., Gil, D., Windover, D., Cline,J. (2011). 
  \pkg{DEoptim:} An R Package for Global Optimization by Differential Evolution. 
  \emph{Journal of Statistical Software}, 40(6), 1-26.
  \doi{10.18637/jss.v040.i06}
}
\keyword{datasets}
